import React from 'react'
import { StyleProvider, Layout, NavLinks, Pagination } from 'mdx-go'
import { LiveProvider, LivePreview, LiveEditor, LiveError } from 'react-live'
import * as Components from '../src'
import { IconList } from '../src/icons'
import tag from 'clean-tag'
import styled, { ThemeProvider } from 'styled-components'
import { StyledSystem } from '../src/components/primitives'
import { theme } from '../src/common/theme'
import * as Icons from 'mdi-react'
const StyledWrapper = styled.div`
  * {
    box-sizing: border-box;
  }
  code[class*='language-'],
  pre[class*='language-'] {
    color: #a9b7c6;
    font-family: Consolas, Monaco, 'Andale Mono', monospace;
    direction: ltr;
    text-align: left;
    white-space: pre;
    word-spacing: normal;
    word-break: normal;
    line-height: 1.5;

    -moz-tab-size: 4;
    -o-tab-size: 4;
    tab-size: 4;

    -webkit-hyphens: none;
    -moz-hyphens: none;
    -ms-hyphens: none;
    hyphens: none;
  }

  pre[class*='language-']::-moz-selection,
  pre[class*='language-'] ::-moz-selection,
  code[class*='language-']::-moz-selection,
  code[class*='language-'] ::-moz-selection {
    color: inherit;
    background: rgba(33, 66, 131, 0.85);
  }

  pre[class*='language-']::selection,
  pre[class*='language-'] ::selection,
  code[class*='language-']::selection,
  code[class*='language-'] ::selection {
    color: inherit;
    background: rgba(33, 66, 131, 0.85);
  }

  /* Code blocks */
  pre[class*='language-'] {
    padding: 1em;
    margin: 0.5em 0;
    overflow: auto;
  }

  :not(pre) > code[class*='language-'],
  pre[class*='language-'] {
    background: #2b2b2b;
  }

  /* Inline code */
  :not(pre) > code[class*='language-'] {
    padding: 0.1em;
    border-radius: 0.3em;
  }

  .token.comment,
  .token.prolog,
  .token.cdata {
    color: #808080;
  }

  .token.delimiter,
  .token.boolean,
  .token.keyword,
  .token.selector,
  .token.important,
  .token.atrule {
    color: #cc7832;
  }

  .token.operator,
  .token.punctuation,
  .token.attr-name {
    color: #a9b7c6;
  }

  .token.tag,
  .token.tag .punctuation,
  .token.doctype,
  .token.builtin {
    color: #e8bf6a;
  }

  .token.entity,
  .token.number,
  .token.symbol {
    color: #6897bb;
  }

  .token.property,
  .token.constant,
  .token.variable {
    color: #9876aa;
  }

  .token.string,
  .token.char {
    color: #6a8759;
  }

  .token.attr-value,
  .token.attr-value .punctuation {
    color: #a5c261;
  }
  .token.attr-value .punctuation:first-child {
    color: #a9b7c6;
  }

  .token.url {
    color: #287bde;
    text-decoration: underline;
  }

  .token.function {
    color: #ffc66d;
  }

  .token.regex {
    background: #364135;
  }

  .token.bold {
    font-weight: bold;
  }

  .token.italic {
    font-style: italic;
  }

  .token.inserted {
    background: #294436;
  }

  .token.deleted {
    background: #484a4a;
  }

  /*code.language-css .token.punctuation {
	color: #cc7832;
}*/

  code.language-css .token.property,
  code.language-css .token.property + .token.punctuation {
    color: #a9b7c6;
  }

  code.language-css .token.id {
    color: #ffc66d;
  }

  code.language-css .token.selector > .token.class,
  code.language-css .token.selector > .token.attribute,
  code.language-css .token.selector > .token.pseudo-class,
  code.language-css .token.selector > .token.pseudo-element {
    color: #ffc66d;
  }
  .prism-code {
    padding: 20px;
    font-family: 'Fira Mono', monospace !important;
    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
    font-size: 15px !important;
    * {
      font-family: 'Fira Mono', monospace !important;
      font-size: 15px !important;
      -webkit-font-smoothing: antialiased;
      -moz-osx-font-smoothing: grayscale;
    }
  }
`
const code = (props) => {
  const isLive = props.className.includes('language-.jsx')
  if (!isLive) return <pre {...props} />
  const code = React.Children.toArray(props.children).join('')

  return (
    <ThemeProvider theme={theme}>
      <LiveProvider
        code={code}
        scope={{
          ...Components,
          tag,
          styled,
          ...StyledSystem,
          ...Icons
        }}
      >
        <Components.Card p={5} bg={'blue.light'} overflow={'hidden'}>
          <Components.Box pb={5}>
            <LivePreview />
          </Components.Box>
          <Components.Box bg={'blue.dark'} overflow="hidden" borderRadius={5}>
            <StyledWrapper>
              <LiveEditor />
            </StyledWrapper>
          </Components.Box>

          <LiveError />
        </Components.Card>
      </LiveProvider>
    </ThemeProvider>
  )
}

export const components = {
  code
}

export const Root = (props) => (
  <StyleProvider components={components} theme={theme}>
    <Layout>
      <Layout.MenuToggle
        style={{
          position: 'fixed'
        }}
      />
      <Layout.Sidebar>
        <NavLinks {...props} />
      </Layout.Sidebar>
      <Layout.Main>
        {props.children}
        <Pagination {...props} />
      </Layout.Main>
    </Layout>
  </StyleProvider>
)
